@@ -17,10 +17,14 @@
 #' Assays with less features won't be reduced in size.
 #' @param fixedAssays A character vector of assay names specifying any assays which must be at the
 #' beginning of the pathway.
-#' @param confidenceCutoff The minimum confidence of predictions for a sample to be predicted by a particular issue
-#' . If a sample was predicted to belong to a particular class a proportion \eqn{p} times, then the confidence is \eqn{2 \times |p - 0.5|}.
+#' @param confidenceCutoff The minimum confidence of predictions for a sample to be predicted by a particular assay. If a sample
+#' was predicted to belong to a particular class a proportion \eqn{p} times, then the confidence is \eqn{2 \times |p - 0.5|}.
 #' @param minAssaySamples An integer specifying the minimum number of samples a tier may have. If a subsequent tier
 #' would have less than this number of samples, the samples are incorporated into the current tier.
+#' @param mode Default: \code{"stability"}. Either \code{"stability"} or \code{"combinatorial"}. If \code{"stability"}, then pathways grow by
+#' passing samples with confidence below \code{confidenceCutoff} to the next assay, until the assays are exhausted or the samples are.
+#' If \code{"combinatorial"}, then all possible combinations of assays respecting \code{fixedAssays}, as well as each assay individually
+#' are considered.
 #' @param nFeatures Default: 20. The number of features to consider during feature selection, if feature selection is done.
 #' @param selectionMethod A named character vector of feature selection methods to use for the assays, one for each. The names must correspond to names of \code{measurements}.
 #' @param classifier A named character vector of modelling methods to use for the assays, one for each. The names must correspond to names of \code{measurements}.
@@ -43,7 +47,7 @@ setGeneric("precisionPathwaysTrain", function(measurements, class, ...)
 #' @export
 setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList", 
           function(measurements, class, useFeatures = NULL, maxMissingProp = 0.0, topNvariance = NULL,
-                   fixedAssays = "clinical", confidenceCutoff = 0.8, minAssaySamples = 10,
+                   fixedAssays = "clinical", confidenceCutoff = 0.8, minAssaySamples = 10, mode = c("stability", "combinatorial"),
                    nFeatures = 20, selectionMethod = setNames(c("none", rep("t-test", length(measurements))), c("clinical", names(measurements))),
                    classifier = setNames(c("elasticNetGLM", rep("randomForest", length(measurements))), c("clinical", names(measurements))),
                    nFolds = 5, nRepeats = 20, nCores = 1)
@@ -56,10 +60,11 @@ setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList",
             }
             if(is.null(useFeatures) && fixedAssays == "clinical")
             {
-              warning("No 'useFeatures' named list element for clincal data is specified. Clinical data often has\n", 
+              warning("No 'useFeatures' named list element for clinical data is specified. Clinical data often has\n", 
     "lots of uninformative variables. Please consider specifying useful features.")        
               useFeatures <- list(clinical = colnames(MultiAssayExperiment::colData(measurements)))
             }
+            mode <- match.arg(mode)
               
             prepArgs <- list(measurements, outcomeColumns = class, useFeatures = useFeatures,
                              maxMissingProp = maxMissingProp, topNvariance = topNvariance)
@@ -67,7 +72,7 @@ setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList",
               
             .precisionPathwaysTrain(measurementsAndClass[["measurements"]], measurementsAndClass[["outcome"]],
                                    useFeatures = useFeatures, fixedAssays = fixedAssays, confidenceCutoff = confidenceCutoff,
-                                   minAssaySamples = minAssaySamples, nFeatures = nFeatures,
+                                   minAssaySamples = minAssaySamples, mode = mode, nFeatures = nFeatures,
                                    selectionMethod = selectionMethod, classifier = classifier,
                                    nFolds = nFolds, nRepeats = nRepeats, nCores = nCores)
           })
@@ -75,16 +80,38 @@ setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList",
 # Internal method which carries out all of the processing, obtaining reformatted data from the
 # MultiAssayExperiment and list (of basic rectangular tables) S4 methods.
 .precisionPathwaysTrain <- function(measurements, class, fixedAssays = "clinical",
-                   useFeatures = useFeatures, confidenceCutoff = 0.8, minAssaySamples = 10,
-                   nFeatures = 20, selectionMethod = setNames(c(NULL, rep("t-test", length(measurements))), c("clinical", names(measurements))),
-                   classifier = setNames(c("elasticNetGLM", rep("randomForest", length(measurements))), c("clinical", names(measurements))),
+                   useFeatures = useFeatures, confidenceCutoff = 0.8, minAssaySamples = 10, mode,
+                   nFeatures = 20, selectionMethod = setNames(c("none", rep("t-test", length(setdiff(unique(mcols(measurements)$assay), "clinical")))), c("clinical", setdiff(unique(mcols(measurements)$assay), "clinical"))),
+                   classifier = setNames(c("elasticNetGLM", rep("randomForest", length(setdiff(unique(mcols(measurements)$assay), "clinical")))), c("clinical", setdiff(unique(mcols(measurements)$assay), "clinical"))),
                    nFolds = 5, nRepeats = 20, nCores = 1)
           {
             # Step 1: Determine all valid permutations of assays, taking into account the
             # assays to be used and which assays, if any, must be included.
             assayIDs <- unique(S4Vectors::mcols(measurements)[["assay"]])
             assaysPermutations <- .permutations(assayIDs, fixed = data.frame(seq_along(fixedAssays), fixedAssays))
-            permutationIDs <- apply(assaysPermutations, 2, function(permutation) paste(permutation, collapse = '-'))
+            assaysPermutations <- asplit(assaysPermutations, 2)
+            if(mode == "combinatorial")
+            {
+                subsetPermutations <- list()
+                variableAssays <- setdiff(assayIDs, fixedAssays)
+                nVariable <- 1
+                while((length(fixedAssays) + nVariable) < length(fixedAssays) + length(variableAssays))
+                {
+                    subsetCombinations <- combn(variableAssays, nVariable)
+                    subsetPermutations <- c(subsetPermutations, apply(subsetCombinations, 2, function(combination)
+                    {
+                        permutations <- .permutations(combination, NULL)
+                        if(is.matrix(permutations)) permutations <- asplit(permutations, 2)
+                        lapply(permutations, function(combinationPermutation) c(fixedAssays, combinationPermutation))
+                    }))
+                    nVariable <- nVariable + 1
+                }
+                subsetPermutations <- unlist(subsetPermutations, recursive = FALSE)
+                assaysPermutations <- c(assaysPermutations, subsetPermutations) # All subsets.
+                assaysPermutations <- c(assaysPermutations, assayIDs) # Each assay on its own.
+            }
+
+            permutationIDs <- lapply(assaysPermutations, function(permutation) paste(permutation, collapse = '-'))
             
             # Step 2: Build a classifier for each assay using all of the samples.
             modelsList <- crossValidate(measurements, class, nFeatures, selectionMethod,
@@ -94,7 +121,7 @@ setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList",
 
             # Step 3: Loop over each pathway and each assay in order to determine which samples are used at that level
             # and which are passed onwards.
-            precisionPathways <- lapply(as.data.frame(assaysPermutations), function(permutation)
+            precisionPathways <- lapply(assaysPermutations, function(permutation)
             {
               assaysProcessed <- character()
               samplesUsed <- character()
@@ -114,13 +141,14 @@ setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList",
                 
                 # Check if too few samples left for next round. Include them in this round, if so.
                 remainingIDs <- setdiff(allSampleIDs, c(samplesUsed, sampleIDsUse))
-                if(length(remainingIDs) < minAssaySamples)
+                if(length(remainingIDs) < minAssaySamples || assay == permutation[length(permutation)])
                 {
                   sampleIDsUse <- c(sampleIDsUse, remainingIDs)
                   breakEarly = TRUE
                 }
+                if(length(sampleIDsUse) == 0) next # No samples have sufficient confidence at this tier.
                 
-                predictionsSamplesCounts <- predictionsSamplesCounts[sampleIDsUse, ]
+                predictionsSamplesCounts <- predictionsSamplesCounts[sampleIDsUse, , drop = FALSE]
                 
                 # Step 3b: Individuals predictions and sample-wise accuracy, tier-wise error.
                 maxVotes <- apply(predictionsSamplesCounts, 1, function(sample) which.max(sample))
@@ -148,10 +176,12 @@ setMethod("precisionPathwaysTrain", "MultiAssayExperimentOrList",
               list(pathway = pathwayString,
                   individuals = individualsTableAll, tiers = tierTableAll)
             })
+
             names(precisionPathways) <- sapply(precisionPathways, "[[", "pathway")
+            attr(modelsList, "crossValParams") <- generateCrossValParams(nRepeats, nFolds, nCores, NULL) 
             precisionPathways <- precisionPathways[unique(names(precisionPathways))] # In case early termination causes duplicates.
             result <- list(models = modelsList, assaysPermutations = assaysPermutations,
-                           parameters = list(confidenceCutoff = confidenceCutoff, minAssaySamples = minAssaySamples),
+                           parameters = list(confidenceCutoff = confidenceCutoff, minAssaySamples = minAssaySamples, classifier = classifier),
                            useFeatures = useFeatures, pathways = precisionPathways)
             class(result) <- "PrecisionPathways"
             
@@ -192,17 +222,36 @@ setMethod("precisionPathwaysPredict", c("PrecisionPathways", "MultiAssayExperime
 
 .precisionPathwaysPredict <- function(pathways, measurements, class)
 {
-
-  # Step 1: Extract all of previously fitted models and permutations.
+  # Step 1: Extract all of previously fitted models and permutations. Create new predictions per assay.
   modelsList <- pathways[["models"]]
   assayIDs <- lapply(pathways[["models"]], function(model) model@characteristics[model@characteristics[, 1] == "Assay Name", 2])
-  assaysPermutations <- pathways[["assaysPermutations"]]
+  assaysPermutations <- strsplit(names(pathways[["pathways"]]), '-')
   confidenceCutoff <- pathways[["parameters"]][["confidenceCutoff"]]
   minAssaySamples <- pathways[["parameters"]][["minAssaySamples"]]
+  classifiers <- pathways[["parameters"]][["classifier"]]
+  allSampleIDs <- rownames(measurements)
   
+  resultsNew <- mapply(function(fittedModel, classifierID)
+  {
+    trainParams <- TrainParams("previousTrained", classifyResult = fittedModel, intermediate = ".iteration")
+    predictPrams <- PredictParams(classifierID)
+    modelParams <- ModellingParams(selectParams = NULL, trainParams = trainParams, predictParams = predictPrams)
+    resultNew <- runTests(measurements, class, attr(modelsList, "crossValParams"), modelParams, verbose = 0)
+    resultNew <- calcCVperformance(resultNew, "Sample Accuracy")
+    predictsNew <- predictions(resultNew)
+    predictionsSamplesCounts <- table(predictsNew[, "sample"], predictsNew[, "class"])
+    sampleAccuracies <- performance(resultNew)[["Sample Accuracy"]]
+    confidences <- 2 * abs(predictionsSamplesCounts[, 1] / rowSums(predictionsSamplesCounts) - 0.5)
+    list(predictionsSamplesCounts, confidences, sampleAccuracies)
+  }, modelsList, classifiers, SIMPLIFY = FALSE)
+  predictionsSamplesCounts <- lapply(resultsNew, "[[", 1)
+  confidences <- lapply(resultsNew, "[[", 2)
+  sampleAccuracies <- lapply(resultsNew, "[[", 3)
+  names(predictionsSamplesCounts) <- names(confidences) <- names(sampleAccuracies) <- assayIDs
+
   # Step 2: Loop over each pathway and each assay in order to determine which samples are used at that level
   # and which are passed onwards.
-  precisionPathways <- lapply(as.data.frame(assaysPermutations), function(permutation)
+  precisionPathways <- lapply(assaysPermutations, function(permutation)
   {
     assaysProcessed <- character()
     samplesUsed <- character()
@@ -211,35 +260,33 @@ setMethod("precisionPathwaysPredict", c("PrecisionPathways", "MultiAssayExperime
     breakEarly = FALSE
     for(assay in permutation)
     {
-      # Step 2a: Identify all samples which are consistently predicted.
-      modelIndex <- match(assay, assayIDs)
-      allPredictions <- predictions(modelsList[[modelIndex]])
-      allSampleIDs <- sampleNames(modelsList[[modelIndex]])
-      predictionsSamplesCounts <- table(allPredictions[, "sample"], allPredictions[, "class"])
-      confidences <- 2 * abs(predictionsSamplesCounts[, 1] / rowSums(predictionsSamplesCounts) - 0.5)
-      sampleIDsUse <- names(confidences)[confidences > confidenceCutoff]
+      # Step 2a: Identify new samples which are consistently predicted.
+      confidencesUse <- confidences[[assay]]
+      predictionsSamplesCountsUse <- predictionsSamplesCounts[[assay]]
+      sampleIDsUse <- names(confidencesUse)[confidencesUse > confidenceCutoff]
       sampleIDsUse <- setdiff(sampleIDsUse, samplesUsed)
                 
       # Check if too few samples left for next round. Include them in this round, if so.
       remainingIDs <- setdiff(allSampleIDs, c(samplesUsed, sampleIDsUse))
-      if(length(remainingIDs) < minAssaySamples)
+      if(length(remainingIDs) < minAssaySamples || assay == permutation[length(permutation)])
       {
         sampleIDsUse <- c(sampleIDsUse, remainingIDs)
         breakEarly = TRUE
       }
+      if(length(sampleIDsUse) == 0) next # No samples have sufficient confidence at this tier.
                 
-      predictionsSamplesCounts <- predictionsSamplesCounts[sampleIDsUse, ]
+      predictionsSamplesCountsUse <- predictionsSamplesCountsUse[sampleIDsUse, , drop = FALSE]
                 
       # Step 2b: Individuals predictions and sample-wise accuracy, tier-wise error.
-      maxVotes <- apply(predictionsSamplesCounts, 1, function(sample) which.max(sample))
-      predictedClasses <- factor(colnames(predictionsSamplesCounts)[maxVotes],
-                                 levels = colnames(predictionsSamplesCounts))    
+      maxVotes <- apply(predictionsSamplesCountsUse, 1, function(sample) which.max(sample))
+      predictedClasses <- factor(colnames(predictionsSamplesCountsUse)[maxVotes],
+                                 levels = colnames(predictionsSamplesCountsUse))
       individualsTable <- S4Vectors::DataFrame(Tier = assay,
                                                `Sample ID` = sampleIDsUse,
                                                `Predicted` = predictedClasses,
-                                               `Accuracy` = performance(modelsList[[modelIndex]])[["Sample Accuracy"]][sampleIDsUse],
+                                               `Accuracy` = sampleAccuracies[[assay]][sampleIDsUse],
                                                 check.names = FALSE)
-      knownClasses <- actualOutcome(modelsList[[modelIndex]])[match(sampleIDsUse, allSampleIDs)]
+      knownClasses <- class[match(sampleIDsUse, allSampleIDs)]
       balancedAccuracy <- calcExternalPerformance(knownClasses, predictedClasses)
       tierTable <- S4Vectors::DataFrame(Tier = assay,
                                         `Balanced Accuracy` = balancedAccuracy, check.names = FALSE)
@@ -257,9 +304,10 @@ setMethod("precisionPathwaysPredict", c("PrecisionPathways", "MultiAssayExperime
          individuals = individualsTableAll, tiers = tierTableAll)
   })
   names(precisionPathways) <- sapply(precisionPathways, "[[", "pathway")
+  precisionPathways <- precisionPathways[unique(names(precisionPathways))]
   result <- list(models = modelsList, assaysPermutations = assaysPermutations,
                  parameters = list(confidenceCutoff = confidenceCutoff, minAssaySamples = minAssaySamples),
-                 pathways = precisionPathways)
+                 pathways = precisionPathways, testSampleInfo = DataFrame(sampleID = rownames(measurements), class = class))
   class(result) <- "PrecisionPathways"
             
   result
@@ -279,12 +327,19 @@ calcCostsAndPerformance <- function(precisionPathways, costs = NULL)
 {
   if(is.null(costs))
     stop("'costs' of each assay must be specified.")
+    
   pathwayIDs <- names(precisionPathways[["pathways"]])
+  if("testSampleInfo" %in% names(precisionPathways))
+  {
+      knownClasses <- precisionPathways[["testSampleInfo"]][, "class"]
+      allNames <- precisionPathways[["testSampleInfo"]][, "sampleID"]
+  } else{
+      knownClasses <- actualOutcome(precisionPathways$models[[1]])
+      allNames <- sampleNames(precisionPathways$models[[1]])      
+  }
   accuraciesCosts <- do.call(rbind, lapply(precisionPathways[["pathways"]], function(pathway)
   {
     predictions <- pathway[["individuals"]][, "Predicted"]
-    knownClasses <- actualOutcome(precisionPathways$models[[1]])
-    allNames <- sampleNames(precisionPathways$models[[1]])
     knownClasses <- knownClasses[match(pathway[["individuals"]][, "Sample ID"], allNames)]
     balancedAccuracy <- calcExternalPerformance(knownClasses, predictions)
     costTotal <- sum(costs[na.omit(match(pathway[["individuals"]][, "Tier"], names(costs)))])
@@ -324,6 +379,7 @@ summary.PrecisionPathways <- function(object, weights = c(accuracy = 0.5, cost =
   if(!is(finalScores, "tabular")) finalScores <- matrix(finalScores, nrow = 1)
   finalScores <- rowSums(finalScores)
   summaryTable <- cbind(summaryTable, Score = finalScores)
+  summaryTable <- summaryTable[order(summaryTable[, "Score"], decreasing = TRUE), ]
   summaryTable
 }
 
@@ -345,7 +401,8 @@ bubblePlot.PrecisionPathways <- function(precisionPathways, pathwayColours = NUL
   performance <- precisionPathways[["performance"]]
   performance <- cbind(Sequence = rownames(performance), performance)
   ggplot2::ggplot(performance, aes(x = accuracy, y = cost, colour = Sequence, size = 4)) + ggplot2::geom_point() +
-    ggplot2::scale_color_manual(values = pathwayColours) + ggplot2::labs(x = "Balanced Accuracy", y = "Total Cost") + ggplot2::guides(size = "none")
+    ggplot2::scale_color_manual(values = pathwayColours) + ggplot2::labs(x = "Balanced Accuracy", y = "Total Cost") +
+    ggplot2::guides(size = "none") + ggplot2::scale_x_continuous(limits = c(0.5, NA)) + ggplot2::scale_y_continuous(limits = c(0, NA))
 }
 
 #' @export
@@ -356,14 +413,19 @@ flowchart <- function (precisionPathways, ...) {
 
 #' @param precisionPathways A pathway of class \code{PrecisionPathways}.
 #' @param pathway A character vector of length 1 specifying which pathway to plot, e.g. "clinical-mRNA".
+#' @param orientation Default: \code{"horizontal"}. Either \code{"horizontal"} or \code{"vertical"}. Specifies the layout of the flowchart.
 #' @param nodeColours A named vector of colours with names being \code{"assay"}, \code{"class1"},\code{"class2"}.
 #' a default colour scheme will automatically be chosen.
 #' @rdname precisionPathwaysEvaluations
 #' @export
-flowchart.PrecisionPathways <- function(precisionPathways, pathway, nodeColours = c(assay = "#86C57C", class1 = "#ACCEE0", class2 = "#F47F72"), ...)
+flowchart.PrecisionPathways <- function(precisionPathways, pathway,
+                                        orientation = c("horizontal", "vertical"),
+                                        nodeColours = c(assay = "snow3", class1 = "#3F48CC", class2 = "#880015"), ...)
 {
   if(!requireNamespace("data.tree", quietly = TRUE))
     stop("The package 'data.tree' could not be found. Please install it.")
+  orientation <- match.arg(orientation)
+  orientation <- ifelse(orientation == "horizontal", "LR", "TB")
 
   pathwayUse <- precisionPathways[["pathways"]][[pathway]]
   assayIDs <- pathwayUse[["tiers"]][, 1]
@@ -376,7 +438,8 @@ flowchart.PrecisionPathways <- function(precisionPathways, pathway, nodeColours
   {
     toDo <- nrow(samplesTiers) - max(which(samplesTiers[, "Tier"] == assay))
     class1Predictions <- currentNode$AddChild(possibleClasses[1], counter = nrow(subset(samplesTiers, Predicted == possibleClasses[1] & Tier == assay)), nodeType = "Class1")
-    uncertain <- currentNode$AddChild("Uncertain", counter = toDo, nodeType = "Uncertain")
+    if(assay != assayIDs[length(assayIDs)])
+      uncertain <- currentNode$AddChild("Uncertain", counter = toDo, nodeType = "Uncertain")
     class2Predictions = currentNode$AddChild(possibleClasses[2], counter = nrow(subset(samplesTiers, Predicted == possibleClasses[2] & Tier == assay)), nodeType = "Class2")
     
     if(toDo == 0) {break} else {
@@ -411,10 +474,10 @@ flowchart.PrecisionPathways <- function(precisionPathways, pathway, nodeColours
     return(label)
   }
           
-  data.tree::SetGraphStyle(pathwayTree, rankdir = "LR")
+  data.tree::SetGraphStyle(pathwayTree, rankdir = orientation)
   data.tree::SetEdgeStyle(pathwayTree, fontname = 'helvetica', label = .getEdgeLabel)
   data.tree::SetNodeStyle(pathwayTree, style = "filled", shape = .getNodeShape, fontcolor = "black", fillcolor = .getFillColour, fontname = 'helvetica')
-  plot(pathwayTree)
+  pathwayTree
 }
 
 .getNodeShape <- function(node){
@@ -435,14 +498,23 @@ strataPlot <- function (precisionPathways, ...) {
 #' a default colour scheme will automatically be chosen.
 #' @rdname precisionPathwaysEvaluations
 #' @export
-strataPlot.PrecisionPathways <- function(precisionPathways, pathway, classColours = c(class1 = "#4DAF4A", class2 = "#984EA3"), ...)
+strataPlot.PrecisionPathways <- function(precisionPathways, pathway, classColours = c(class1 = "#3F48CC", class2 = "#880015"), ...)
 {
   if(missing(pathway)) stop("'pathway' is not specified. Please specify one of the ones shown by printing the trained object.")    
   pathwayUse <- precisionPathways[["pathways"]][[pathway]]
   assayIDs <- pathwayUse[["tiers"]][, 1]
   possibleClasses <- levels(precisionPathways$models[[1]]@actualOutcome)
   samplesTiers <- pathwayUse$individuals
-  samplesTiers$trueClass <- actualOutcome(precisionPathways$models[[1]])[match(samplesTiers[, "Sample ID"], sampleNames(precisionPathways$models[[1]]))]
+  
+  if("testSampleInfo" %in% names(precisionPathways))
+  {
+    knownClasses <- precisionPathways[["testSampleInfo"]][, "class"]
+    allNames <- precisionPathways[["testSampleInfo"]][, "sampleID"]
+    samplesTiers$trueClass <- knownClasses[match(samplesTiers[, "Sample ID"], allNames)]
+  } else {
+    samplesTiers$trueClass <- actualOutcome(precisionPathways$models[[1]])[match(samplesTiers[, "Sample ID"], sampleNames(precisionPathways$models[[1]]))]     
+  }
+  
   samplesTiers <- dplyr::arrange(as.data.frame(samplesTiers), Tier, trueClass, Accuracy)
   samplesTiers$ID = 1:nrow(samplesTiers)
   samplesTiers$colour = ifelse(samplesTiers$trueClass == levels(samplesTiers[, "Predicted"])[1], classColours["class1"], classColours["class2"])
@@ -454,7 +526,7 @@ strataPlot.PrecisionPathways <- function(precisionPathways, pathway, classColour
     ggplot2::guides(fill = ggplot2::guide_legend(title.position = "top")) +
     ggnewscale::new_scale_fill() +
     ggplot2::geom_tile(ggplot2::aes(fill = Accuracy)) +
-    ggplot2::scale_fill_gradient(low = "#377EB8", high = "#E41A1C") +
+    ggplot2::scale_fill_gradient2(low = "#86C57C", mid = "white", high = "#FFFF28", midpoint = 0.5) +
     ggplot2::labs(fill = "Accuracy") +
     ggplot2::guides(fill = ggplot2::guide_colorbar(title.position = "top")) +
     ggplot2::theme(panel.background = ggplot2::element_blank(),
