@@ -1,12 +1,19 @@
-selectMulti <- function(measurementsTrain, classesTrain, params, verbose = 0)
-          {                      
+selectMulti <- function(measurementsTrain, outcomeTrain, params, verbose = 0)
+          {                 
               assaysIndices <- lapply(unique(S4Vectors::mcols(measurementsTrain)[["assay"]]), function(assay) which(S4Vectors::mcols(measurementsTrain)[["assay"]] == assay))
               assayTrain <- lapply(assaysIndices, function(assayIndices) measurementsTrain[, assayIndices, drop = FALSE])
+              tuneMode <- "none"
+              performanceType <- "N/A"
+              if(!is.null(params[[1]]@selectParams@tuneParams))
+              {
+                  tuneMode <- "Resubstitution"
+                  if(is(outcomeTrain, "Surv")) performanceType <- "C-index" else performanceType <- "Balanced Accuracy"
+              }
               featuresIndices <- mapply(.doSelection, 
                                          measurements = assayTrain,
                                          modellingParams = params,
-                                         MoreArgs = list(outcomeTrain = classesTrain, 
-                                                         crossValParams = CrossValParams(permutations = 1, folds = 5), ###### Where to get this from?
+                                         MoreArgs = list(outcomeTrain = outcomeTrain, 
+                                                         crossValParams = CrossValParams(permutations = 1, folds = 5, tuneMode = tuneMode, performanceType = performanceType), ###### Where to get this from?
                                                          verbose = 0), SIMPLIFY = FALSE
                                         )
 
